********************************************************************************

use psid0414_build, clear

binscatter share_tradeables wealth_rank [aw=weight],  ///
	xtitle("wealth decile", size(large)) ///
	ytitle("tradable expenditure share (PSID)", size(large)) ///
	linetype(lfit) bgcolor(white) ysize(3) xsize(4) ///
	xlabel(1(1)10, nogrid tp(inside) labsize(large)) ///
	ylabel(.25(.05).35, angle(horizontal) nogrid tp(inside) format(%03.2f) labsize(large)) ///
	plotregion(lcolor(black) lwidth(medium) margin(medium) ) ///
	graphregion(color(white)) 
graph export psid_share_by_wealth_rank.pdf, as(pdf) replace

binscatter share_tradeables inc_rank [aw=weight],  ///
	xtitle("disposable labor income decile", size(large)) ///
	ytitle("tradable expenditure share (PSID)", size(large)) ///
	linetype(lfit) bgcolor(white) ysize(3) xsize(4) ///
	xlabel(1(1)10, nogrid tp(inside) labsize(large)) ///
	ylabel(.25(.05).35, angle(horizontal) nogrid tp(inside) format(%03.2f) labsize(large)) ///
	plotregion(lcolor(black) lwidth(medium) margin(medium) ) ///
	graphregion(color(white)) 
graph export psid_share_by_inc_rank.pdf, as(pdf) replace

gen ln_wealth = log(hh_wealth)
gen ln_income = log(hh_lab_inc)
label var ln_wealth "Wealth"
label var ln_income "Income"
replace share_tradeables = 100*share_tradeables

eststo clear
eststo wealth: quietly reg share_tradeables ln_wealth i.year  [aw=weight] 
eststo income: quietly reg share_tradeables ln_income i.year 
eststo both: quietly reg share_tradeables ln_wealth ln_income i.year  [aw=weight] 
eststo both_asc: quietly reg share_tradeables ln_wealth ln_income i.year i.age i.hh_size i.collegegrad i.homeowner [aw=weight] 

esttab, title("PSID Tradeable Share") ///
	keep(ln_* 1.college* 1.home*) ar2 se noconstant star(* 0.10 ** 0.05 *** 0.01) ///
	interaction(×) label nodepvars nomtitles replace ///
	addnotes("(1) and (2) include year fixed effects. (3) includes year, " ///
	"age, family size, education, and home owner fixed effects")
	
*******************************************************************************

use cex0414_build, clear

binscatter share_tradeables_bls wealth_rank [aw=weight],  ///
	xtitle("liquid wealth decile", size(large)) ///
	ytitle("tradable expenditure share (CEX)", size(large)) ///
	linetype(lfit) bgcolor(white) ysize(3) xsize(4) ///
	xlabel(1(1)10, nogrid tp(inside) labsize(large)) ///
	ylabel(.35(.05).45, angle(horizontal) nogrid tp(inside) format(%03.2f) labsize(large)) ///
	plotregion(lcolor(black) lwidth(medium) margin(medium) ) ///
	graphregion(color(white)) 
graph export cex_share_by_wealth_rank.pdf, as(pdf) replace 

binscatter share_tradeables_bls inc_rank [aw=weight],  ///
	xtitle("disposable labor income decile", size(large)) ///
	ytitle("tradable expenditure share (CEX)", size(large)) ///
	linetype(lfit) bgcolor(white) ysize(3) xsize(4) ///
	xlabel(1(1)10, nogrid tp(inside) labsize(large)) ///
	ylabel(.35(.05).45, angle(horizontal) nogrid tp(inside) format(%03.2f) labsize(large)) ///
	plotregion(lcolor(black) lwidth(medium) margin(medium) ) ///
	graphregion(color(white)) 
graph export cex_share_by_inc_rank.pdf, as(pdf) replace 

gen ln_wealth = log(wealth)
gen ln_income = log(income)
label var ln_wealth "Wealth"
label var ln_income "Income"
replace share_tradeables_bls = 100*share_tradeables_bls

eststo clear
eststo wealth: quietly reg share_tradeables_bls ln_wealth i.year [aw=weight]
eststo income: quietly reg share_tradeables_bls ln_income i.year 
eststo both: quietly reg share_tradeables_bls ln_wealth ln_income i.year [aw=weight]
eststo both_asc: quietly reg share_tradeables_bls ln_wealth ln_income i.year i.age i.fam_size i.collegegrad i.homeowner [aw=weight]

esttab, title("CEX Tradeable Share") ///
	keep(ln_* 1.college* 1.home*) ar2 se noconstant star(* 0.10 ** 0.05 *** 0.01) ///
	interaction(×) label nodepvars nomtitles replace ///
	addnotes("(1) and (2) include year fixed effects. (3) includes year, " ///
	"age, family size, education, and home owner fixed effects")
